.area _DATA
.area _CODE

.include "keyboard.h.s"
.include "hero.h.s"
.include "person.h.s"
.include "cpctelera.h.s"

;;Sprites
.globl _person_hater	;;RED 	-> 0
.globl _person_lover	;;GREEN -> 1
.globl _person_hero		;;BLUE  -> 2

;;Hero Data
hero_x: .db #39
hero_y:	.db #80
hero_w: .db #6
hero_h: .db #12
hero_group: .db #2 
hero_died: .db #0


;;==================================================================================
;;New hero
;;==================================================================================
hero_born::
	ld a, #0
	ld (hero_died),  a			;;Not died
	ld a, #2
	ld (hero_group), a			;;Hero blue
	ret
;;==================================================================================
;; Extern call for draw the Hero
;;==================================================================================
hero_draw::
	call drawHero
	ret

;;==================================================================================
;; Extern call for erase the Hero
;;==================================================================================
hero_erase::
		ld a, #0x00
		call drawHero
	ret
;;==================================================================================
;; Extern call for update the Hero
;;==================================================================================
hero_update::
		ld a, #0x00
		call checkUserInput			;;Use awsd keyboard
		call joystick 				;;Use joystick
	ret
;;==================================================================================
;; Get pointer to
;;==================================================================================
hero_getPtrHL::
		ld	hl, #hero_x
	ret

;;==================================================================================
;; Moves Hero to right
;;==================================================================================
moveHeroRight:
	ld a, (hero_x)				;;A = hero_x
	cp #80-5					;;Check against right limit (screen size - hero size)
	jr z, d_not_move_right		;;Hero_x == Limit, do not move
		inc a 					;;A++ (hero_x++)
		ld (hero_x), a 			;;Update hero_x
	d_not_move_right:
	ret
;;==================================================================================
;; Moves Hero to left
;;==================================================================================
moveHeroLeft:
	ld a, (hero_x)			;;A = hero_x
	cp #0					;;Check against left limit (screen size - hero size)
	jr z, d_not_move_left	;;Hero_x == Limit, do not move
		dec a 				;;A-- (hero_x--)
		ld (hero_x), a 		;;Update hero_x
	d_not_move_left:
	ret
;;==================================================================================
;; Moves Hero to up
;;==================================================================================
moveHeroUp:
	ld a, (hero_y)			;;A = hero_y
	cp #0					;;Check against left limit (screen size - hero size)
	jr z, d_not_move_up	;;Hero_y == Limit, do not move
		;;Twice to go faster
		dec a 				;;A-- (hero_y--)
		ld (hero_y), a 		;;Update hero_y
		dec a 				;;A-- (hero_y--)
		ld (hero_y), a 	 	;;Update hero_y
	d_not_move_up:
	ret
;;==================================================================================
;; Moves Hero to down
;;==================================================================================
moveHeroDown:
	ld a, (hero_y)	;;A = hero_y
	cp #200-34		;;Check against left limit (screen size - hero size)
	jr z, d_not_move_down	;;Hero_y == Limit, do not move
	
	;;Twice to go faster
	inc a			;;A++ (hero_y++)
	ld (hero_y), a	;;Update hero_y
	inc a			;;A++ (hero_y++)
	ld (hero_y), a	;;Update hero_y

	d_not_move_down:
	ret
;;==================================================================================
;; Devuelve el valor de hero_x
;;==================================================================================
getHerox::
		ld a, (hero_x)
	ret
;;==================================================================================
;; Devuelve el valor de hero_y
;;==================================================================================
getHeroy::
		ld a, (hero_y)
	ret
;;==================================================================================
;;	Checks User Input and Reacts
;;==================================================================================
checkUserInput:
	;;Scan the whole keyboard
	call cpct_scanKeyboard_asm ;;keyboard.s

	;;Check for key 'D' being presed
	ld hl, #Key_D 				;;HL = Key_D
	call cpct_isKeyPressed_asm	;;Check if Key_D is presed
	cp #0						;;Check d == 0
	jr z, d_not_pressed			;;Jump if d ==0 (d_not_pressed)

	;;D is pressed
	call moveHeroRight
	d_not_pressed:

	;;Check for key 'A' being presed
	ld hl, #Key_A 				;;HL = Key_A
	call cpct_isKeyPressed_asm	;;Check if Key_A is presed
	cp #0						;;Check A == 0
	jr z, a_not_pressed			;;Jump if A==0 (a_not_pressed)

	;;A is pressed
	call moveHeroLeft
	a_not_pressed:

	;;Check for key 'W' being presed
	ld hl, #Key_W 				;;HL = Key_W
	call cpct_isKeyPressed_asm	;;Check if Key_W is presed
	cp #0						;;Check W == 0
	jr z, w_not_pressed			;;Jump if W==0 (w_not_pressed)

	;;W is pressed
	call moveHeroUp
	w_not_pressed:


	;;Check for key 'Z' being presed
	ld hl, #Key_S 				;;HL = Key_S
	call cpct_isKeyPressed_asm	;;Check if Key_S is presed
	cp #0						;;Check S == 0
	jr z, s_not_pressed			;;Jump if S==0 (s_not_pressed)

	;;S is pressed
	call moveHeroDown
	s_not_pressed:

	;;Check for key 'K' being presed
	ld hl, #Key_K				;;HL = Key_K
	call cpct_isKeyPressed_asm	;;Check if Key_K is presed
	cp #0						;;Check K == 0
	jr z, k_not_pressed			;;Jump if K==0 (k_not_pressed)
	;;K is pressed
		ld a, (hero_group)
		cp #0
		jr z, cambiarHater
		cambiarHater:
			ld a, #0
			ld (hero_group), a 
			call hero_draw
		ret
	k_not_pressed:
		ld a, #2
		ld (hero_group), a 
		call hero_draw

	;;Check for key 'L' being presed
	ld hl, #Key_L				;;HL = Key_L
	call cpct_isKeyPressed_asm	;;Check if Key_L is presed
	cp #0						;;Check L == 0
	jr z, l_not_pressed			;;Jump if L==0 (l_not_pressed)
	;;L is pressed
		ld a, (hero_group)
		cp #0
		jr z, cambiarLover
		cambiarLover:
			ld a, #1
			ld (hero_group), a 
			call hero_draw
		ret
	l_not_pressed:
		ld a, #2
		ld (hero_group), a 
		call hero_draw
	ret
;;==================================================================================
;;	Draw the hero
;;	DESTROYS: AF, BC, DE, HL
;;==================================================================================
drawHero:
	push af 		;;Save A in the stack
	;; Calculate Screen position
	ld de, #0xC0F0	;;Video memory

	ld a, (hero_x)	;;|
	ld c, a			;;\ C=hero_x

	ld a, (hero_y)	;;|
	ld b, a			;;\ B=hero_y

	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x0C06
	pop af	
		ld a, (hero_group)
		cp #2
		jr z, printNormal
		cp #1
		jr z, printLover
		cp #0
		jr z, printHater

		;;Draw Hero
		printNormal:
			ld	hl, #_person_hero
			call cpct_drawSprite_asm
		ret

		printHater:
			ld	hl, #_person_hater
			call cpct_drawSprite_asm
		ret

		printLover:
			ld	hl, #_person_lover
			call cpct_drawSprite_asm
		ret
	ret
;;==================================================================================
;;	Checks User Input and Reacts
;;==================================================================================
joystick:
	;;Scan the whole keyboard
	call cpct_scanKeyboard_asm ;;keyboard.s

	;;Check for key 'right' being presed
	ld hl, #Joy0_Right 				;;HL = right
	call cpct_isKeyPressed_asm		;;Check if right is presed
	cp #0							;;Check A == 0
	jr z, noRight					;;Jump if A==0 (noRight)

	;;right is pressed
	call moveHeroRight
	noRight:

	;;Check for key 'left' being presed
	ld hl, #Joy0_Left 				;;HL = left
	call cpct_isKeyPressed_asm		;;Check if left is presed
	cp #0							;;Check A == 0
	jr z, noLeft					;;Jump if A==0 (noLeft)

	;;A is pressed
	call moveHeroLeft
	noLeft:

	;;Check for key 'up' being presed
	ld hl, #Joy0_Up 				;;HL = up
	call cpct_isKeyPressed_asm		;;Check if up is presed
	cp #0							;;Check W == 0
	jr z, noUp						;;Jump if W==0 (noUp)

	;;W is pressed
	call moveHeroUp
	noUp:

	;;Check for key 'down' being presed
	ld hl, #Joy0_Down 				;;HL = down
	call cpct_isKeyPressed_asm		;;Check if down is presed
	cp #0							;;Check W == 0
	jr z, noDown					;;Jump if Z==0 (noDown)

	;;Z is pressed
	call moveHeroDown
	noDown:
	ret
;;==================================================================================
;;Check collisions between hero and people
;;==================================================================================
hero_checkCollision::
	;;Check horizontal colisions (En el ejeX)	
	;;Izquierda
	;; if (hero_x + hero_w < person_x) no_collision  -> hero_x + hero_w - person_x <= 0
	ld	a, (hero_x)					;;A = hero_x
	ld  c, a						;;C = hero_x
	ld  a, (hero_w)					;;A = hero_w
	add c							;;A + C = hero_w + hero_x
	sub 1(ix)						;;IX = person_x -> (hero_w + hero_x) - person_x
	jr z, no_collision				;;if ( (person_w + person_x) - person_x) == 0 ->no_colision
	jp m, no_collision				;;if ( (person_w + person_x) - person_x) < 0  ->no_colision
	
	;;Derecha
	;; if (person_x + person_w <= hero_x) ->  person_x + person_w - hero_x <= 0)
	ld a, 1(ix)						;;A = person_x
	ld c, a							;;C = person_x
	ld a, 3(ix)						;;A = person_w
	add	c							;;A = person_x + person_w
	ld	c, a						;;C = person_x + person_w
	ld 	a, (hero_x)					;;A = hero_x
	ld	b, a						;;B = hero_x
	ld	a, c						;;A = (person_x + person_w)
	sub b							;;A - B -> (person_x + person_w) - hero_x
	jr z, no_collision				;;if ( (person_w + person_x) - person_x) == 0 ->no_colision
	jp m, no_collision				;;if ( (person_w + person_x) - person_x) < 0  ->no_colision
	
	;;Check vertical colisions
	;;Arriba
	;; if (hero_y + hero_h < person_y) no_collision -> ((hero_y + hero_h) - person_y) <= 0
	ld	a, (hero_y)					;;A = hero_y
	ld  c, a						;;C = hero_y
	ld  a, (hero_h)					;;A = hero_h
	add c							;;A + C -> hero_y + hero_h
	ld c, a							;;C = hero_y + hero_h
	ld a, 2(ix)						;;A = person_y
	ld b, a							;;B = person_y
	ld a, c							;;A = hero_y + hero_h
	sub b 							;;A = hero_y + hero_h) - person_y
	jr z, no_collision				;;if ( (person_w + person_x) - person_x) == 0 ->no_colision
	jp m, no_collision				;;if ( (person_w + person_x) - person_x) < 0  ->no_colision

	;;Abajo
	;; if (person_y + person_h <= hero_y) -> ((person_y + person_h) - hero_y) <= 0)
	ld a, 2(ix)					;;A = person_y
	ld c, a						;;C = person_y
	ld a, 4(ix)					;;A = person_h
	add c						;;A = person_y + person_h 
	ld	c, a					;;C = person_y + person_h 
	ld 	a, (hero_y)	  			;;A = person_y
	ld	b, a					;;B = person_y
	ld	a, c					;;A = person_y + person_h
	sub b						;;A = (person_y + person_h) - person_y
	jr z, no_collision			;;if ( (person_w + person_x) - person_x) == 0 ->no_colision
	jp m, no_collision			;;if ( (person_w + person_x) - person_x) < 0  ->no_colision

	;;Collision
	call collision
	ret
	;;No Collision
	no_collision:
	ret
;;==================================================================================
;;Check if the hero is died 0->Not died, 1->Is died
;;==================================================================================
is_hero_died::
		ld a, (hero_died)
	ret
;;==================================================================================
;;Prepare collision
;;==================================================================================
collision:
	ld a, (hero_group)
	cp #2				
	jr nz, notBlue				;;if (hero_group != 2) -> heroActuando
		;;Colision con Hero azul -> Hero muere
		;ld a, #1
		;ld (hero_died), a
		call person_erase
	ret

	notBlue:
		ld a, 6(ix)				;;A = person_group
		ld b, a					;;B = person_group
		ld a, (hero_group)		;;A = hero_group
		cp b
		
		jr z, sonIguales
			call person_erase				;;Borro a la persona
			call person_create_hater		;;Creo un hater
			call person_create_hater				;;Creo un hater
		ret
		sonIguales:		
			call person_erase				;;Borro a la persona
			call person_create_lover		;;Creo un lover
			call person_create_lover		;;Creo un lover
		ret
	ret
