.area _DATA
.area _CODE

.include "keyboard.h.s"
.include "cpctelera.h.s"
.include "hero.h.s"
.include "person.h.s"
.include "meter.h.s"
.include "goal.h.s"

;;Sprites
.globl _person_palette
.globl _person_gameOver
.globl _person_phantastic
.globl _person_pressSpace
.globl _person_nextLevel

.globl _person_patience
.globl _person_welcome
.globl _person_kl
.globl _person_greenBall
.globl _person_redBall
.globl _person_wasd

;;Song
.globl _song_ingame

;;Main data
game_playing: 	.db #0 	 ;;0=no, 1=yes			
level_of_detail:.db #0
value: 			.db #12

;;==================================================================================
;;	Main program entry
;;==================================================================================
_main::
	call initializeGame			;;|
	call initializeMatch		;;|
	call startGame				;;\ Initial process

	main_loop:
		;;ERASE		
		call meter_erase			;;Erase the meter
		call people_eraseCollisions	;;Erase people

		;;level of detail control
		call levelDetail
		
		;;RULES
		call gameOverRules
		call winnerRules

		call playerReestart
		ld a, (game_playing)
		cp #0
		jr z, main_loop		

		;;UPDATE
		call hero_update
		ld a, (level_of_detail)		;;|
		cp #0						;;|
		jr z, noUpdate				;;|
			call people_update		;;|
			call meter_update		;;\Level of detail control
			call goal_update
		noUpdate:

		;;DRAW
		call hero_draw
		call people_draw
		call drawGoals
		call meter_draw

		;;WAIT
		call cpct_waitVSYNC_asm
	jr main_loop
	ret


;;==================================================================================
;;INITIALIZE GAME
;;Firmaware actions
;;==================================================================================
initializeGame:
	call cpct_disableFirmware_asm 				;;|Firmware interruption
	ld	c, #0									;;|Modo 0
	call cpct_setVideoMode_asm					;;\Cambiar modo video

	ld 	hl, #_person_palette					;;|Seleccionar paleta 
	ld 	de, #16									;;| de 16 colores
	call cpct_setPalette_asm					;;\ Cambiar paleta

	ld de, #_song_ingame						;;|			
	call cpct_akp_musicInit_asm					;;\Music control

	ld hl, #interruptions						;;|
	call cpct_setInterruptHandler_asm			;;\Interruptions

    ret
;;==================================================================================
;;Control the interruptions
;;==================================================================================
interruptions::
	ex af, af'
	exx
	push af
	push bc
	push de
	push hl
	push iy

	call cpct_scanKeyboard_if_asm

	ld a, (value)
	dec a
	ld (value), a
	jr nz, return

		call cpct_akp_musicPlay_asm
		ld a, #12
		ld (value), a

	return:
	pop iy
	pop hl
	pop de
	pop bc
	pop af
	exx
	ex af, af'
	
	ret
;;==================================================================================
;;INITIALIZE Match
;;==================================================================================
initializeMatch:
	call screenErase							;;Clean the screen

	call hero_born								;;New hero
	
	call people_kill_everyone					;;Reestart people
	
	call person_create_lover					;;Creo un lover
	call person_create_hater					;;Creo un hater

	call meter_create							;;Create the meter

	call goal_create							;;Create the goals

	ld a, #1									;;|
	ld (game_playing), a						;;\ Start the game
	ret
;;==================================================================================
;;Control level of detail
;; if lod = 1 -> 0 else ->1
;;==================================================================================
levelDetail:
	ld a, (level_of_detail)
	cp #0
	jr z, cambiar
		ld a, #0
		ld (level_of_detail), a
	ret
	cambiar:
		ld a, #1
		ld (level_of_detail), a
	ret
;;==================================================================================
;;Clean the screen
;;==================================================================================
screenErase:
	;; Calculate Screen position
	ld de, #0xC000				;;Video memory

	ld a, #0					;;|
	ld c, a						;;\ C = meter_x
	ld a, #0					;;|
	ld b, a						;;\ B = meter_y

	call cpct_getScreenPtr_asm	;;Get pointer to screen
	ex de, hl
	;; Draw a box
	ld a, #0x00					;;PALETTE Black
	ld bc, #0xFF40				;;80x200 -> 200x80 -> 100x80
	call cpct_drawSolidBox_asm

	;; Calculate Screen position
	ld de, #0xC000				;;Video memory

	ld a, #50					;;|
	ld c, a						;;\ C = meter_x
	ld a, #0					;;|
	ld b, a						;;\ B = meter_y

	call cpct_getScreenPtr_asm	;;Get pointer to screen
	ex de, hl
	;; Draw a box
	ld a, #0x00					;;PALETTE Black
	ld bc, #0xFF40				;;80x200 -> 200x80 -> 100x80
	call cpct_drawSolidBox_asm
	ret	
;;==================================================================================
;;Check if the player wants to play again
;;==================================================================================
playerReestart:
	;;Check for key 'L' being presed
	call cpct_scanKeyboard_asm
	ld hl, #Key_Space				;;HL = Key_L
	call cpct_isKeyPressed_asm		;;Check if Key_L is presed
	cp #0							;;Check L == 0
	jr z, space_not_pressed			;;Jump if L==0 (w_not_pressed)
		call initializeMatch
	space_not_pressed:
	ret		
;;==================================================================================
;;Game Over Rules
;;==================================================================================
gameOverRules:

	;;Hero die
	ld a, #0
	call is_hero_died					;;Check if the hero is died
	cp #0
	jr z, notHeroDied					;;|if (hero_died == 1) -> Reestart game	
		ld a, #0						;;|
		ld (game_playing), a			;;| game_playing = 0
		call drawGameOver				;;\ Draw Game Over Sprite
		call drawReestart 
		call playerReestart
	ret

	notHeroDied:
	;;All people die
	call people_died				;;|Check if everyone is died
	cp #0							;;|				
	jp z, notEveryoneDied			;;|if (a == 0) -> At least one alive 
		ld a, #0
		ld (game_playing), a
		call drawGameOver			;;\Draw Game Over Sprite
		call drawReestart 
		call playerReestart
	ret
	notEveryoneDied:
	ret	

;;==================================================================================
;;Check if the user is winner
;;==================================================================================
winnerRules:
	call isWinner
	cp #0
	jr z, noWinner
		ld a, #0						;;|
		ld (game_playing), a			;;| game_playing = 0
		call drawWinner					;;|
		call playerReestart				;;\Draw winner sprite
	ret
	noWinner:
	ret

;;==================================================================================
;;Draw the Game Over Sprite
;;==================================================================================
drawGameOver:
	ld de, #0xC000				;;Video memory	
	ld b, #30					;;Y
	ld c, #100					;;X		
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_gameOver	;;Game Over Sprite
	call cpct_drawSprite_asm	;;Draw
	ret	
;;==================================================================================
;;Draw a Sprite for winners
;;==================================================================================
drawWinner:
	ld de, #0xC000				;;Video memory	
	ld b, #60					;;X
	ld c, #100					;;Y		
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_phantastic	;;Draw Winner Sprite
	call cpct_drawSprite_asm	;;Draw
	ret
;;==================================================================================
;;Draw a Sprite press space
;;==================================================================================
drawReestart:
	ld de, #0xC000				;;Video memory	
	ld b, #80					;;Y
	ld c, #100					;;X	
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_pressSpace ;;Press Space Sprite
	call cpct_drawSprite_asm	;;Draw
	ret
;;==================================================================================
;;Draw a Sprite Next Level
;;==================================================================================
drawNextLevel::
	ld de, #0xC000				;;Video memory	
	ld b, #80					;;Y
	ld c, #100					;;X	
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_nextLevel  ;;Press Space Sprite
	call cpct_drawSprite_asm	;;Draw
	call wait
	call wait
	call screenEraseLevel
	ret

;;==================================================================================
;;Clean the screen
;;==================================================================================
screenEraseLevel:
	;; Calculate Screen position
	ld de, #0xC000				;;Video memory

	ld a, #100					;;|
	ld c, a						;;\ C = meter_x
	ld a, #80					;;|
	ld b, a						;;\ B = meter_y

	call cpct_getScreenPtr_asm	;;Get pointer to screen
	ex de, hl
	;; Draw a box
	ld a, #0x00					;;PALETTE Black
	ld bc, #0x3228				;;80x200 -> 200x80 -> 100x80
	call cpct_drawSolidBox_asm
	ret
;;==================================================================================
;;Wait some time
;;==================================================================================
wait:
	ld b, #0xFF
	wLoop:
	ld a, b
	cp #0
		ret z
    ld a, #0
    waitLopp:
        cp #0xFF
            jr z, decrement
        inc a
        jr waitLopp

		decrement:
		ld a, #0
		dec b
		jr wLoop
	ret

;;==================================================================================
;;Show the menu
;;==================================================================================
startGame:
	call drawWelcome		;;|
	call drawPatience		;;|
			call wait 		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
		call screenErase	;;\Draw the first screen

	call drawWasd			;;|
	call drawKl				;;|
			call wait 		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
		call screenErase	;;\Draw the second screen

	call drawGreen			;;|
	call drawRed			;;|
			call wait 		;;|
			call wait		;;|
			call wait		;;|	
			call wait		;;|
			call wait		;;|
			call wait		;;|
			call wait		;;|
		call screenErase	;;\Draw the third screen
	ret
;;==================================================================================
;;Draw welcome Sprite
;;==================================================================================
drawWelcome:
	ld de, #0xC000				;;Video memory	
	ld b, #30					;;Y
	ld c, #100					;;X		
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_welcome	;;Game Over Sprite
	call cpct_drawSprite_asm	;;Draw
	ret	
;;==================================================================================
;;Draw patience Sprite
;;==================================================================================
drawPatience:
	ld de, #0xC000				;;Video memory	
	ld b, #80					;;Y
	ld c, #100					;;X	
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_patience ;;Press Space Sprite
	call cpct_drawSprite_asm	;;Draw
	ret
;;==================================================================================
;;Draw WASD Sprite
;;==================================================================================
drawWasd:
	ld de, #0xC000				;;Video memory	
	ld b, #30					;;Y
	ld c, #100					;;X		
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_wasd	;;Game Over Sprite
	call cpct_drawSprite_asm	;;Draw
	ret	
;;==================================================================================
;;Draw KL Sprite
;;==================================================================================
drawKl:
	ld de, #0xC000				;;Video memory	
	ld b, #80					;;Y
	ld c, #100					;;X	
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_kl ;;Press Space Sprite
	call cpct_drawSprite_asm	;;Draw
	ret
;;==================================================================================
;;Draw Red Balls
;;==================================================================================
drawRed:
	ld de, #0xC000				;;Video memory	
	ld b, #30					;;Y
	ld c, #100					;;X		
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_redBall	;;Game Over Sprite
	call cpct_drawSprite_asm	;;Draw
	ret	
;;==================================================================================
;;Draw Green Balls Sprite
;;==================================================================================
drawGreen:
	ld de, #0xC000				;;Video memory	
	ld b, #80					;;Y
	ld c, #100					;;X	
	call cpct_getScreenPtr_asm	;;Get pointer to screen

	ex de, hl
	ld bc, #0x3228 				;;REAL 80x50 -> 40x50 -> 50x40 -> 32x28
	ld	hl, #_person_greenBall ;;Press Space Sprite
	call cpct_drawSprite_asm	;;Draw
	ret